@@ -22,20 +22,22 @@
 #include "xbuffy.h"
 
 #define cmpTok(tok,str,tokVal) {if ( strcmp(tok,str) == 0) return(tokVal);}
+#define checkAndFree(x) {if (x!=NULL) {free(x);x=NULL;}}
 
 extern ApplicationData_t data;
 
-static char *tokens[20] = {
+static char *tokens[23] = {
 	"box", "title", "command", "audio", "mailbox", "newsbox", "origmode",
 	"newmode", "beep", "nobeep", "last", "headertime", "polltime",
-	"shortname", "longname", "background", "foreground", NULL
+        "shortname", "longname", "background", "foreground", 
+        "cclient", "countperiod", "keepopen", NULL
 };
 
 enum TokType
 {
 	UNK_T = 0, BOX_T, TITLE_T, COMMAND_T, AUDIO_T, MAILBOX_T, NEWSBOX_T,
 	ORIGMODE_T, NEWMODE_T, BEEP_T, NOBEEP_T, LAST_T, HEADER_T, POLL_T,
-	SHORT_T, LONG_T, BG_T, FG_T
+	SHORT_T, LONG_T, BG_T, FG_T, CCLIENT_T, COUNTPERIOD_T, KEEPOPEN_T
 };
 
 typedef enum TokType TokenType;
@@ -81,49 +83,31 @@ TokenType token(line, next)
 void clearBox(tempBox)
 	BoxInfo_t *tempBox;
 {
-	if (tempBox->box != NULL)
-	{
-		free(tempBox->box);
-		tempBox->box = NULL;
-	}
-
-	if (tempBox->command != NULL)
-	{
-		free(tempBox->command);
-		tempBox->command = NULL;
-	}
-
-	if (tempBox->audioCmd != NULL)
-	{
-		free(tempBox->audioCmd);
-		tempBox->audioCmd = NULL;
-	}
-
-	if (tempBox->boxTitle != NULL)
-	{
-		free(tempBox->boxTitle);
-		tempBox->boxTitle = NULL;
-	}
-   
-   	if (tempBox->bgName != NULL)
-	{
-		free(tempBox->bgName);
-		tempBox->bgName = NULL;
-	}
-
-      	if (tempBox->fgName != NULL)
-	{
-		free(tempBox->fgName);
-		tempBox->fgName = NULL;
-	}
-
-   
+        checkAndFree(tempBox->box);
+	checkAndFree(tempBox->command);
+	checkAndFree(tempBox->audioCmd);
+	checkAndFree(tempBox->boxTitle);
+	checkAndFree(tempBox->bgName);
+	checkAndFree(tempBox->fgName);
+        checkAndFree(tempBox->stream);
+        checkAndFree(tempBox->uname);
+        checkAndFree(tempBox->passwd);
 
 	tempBox->type = 0;
 	tempBox->last = 0;
 	tempBox->headerTime = tempBox->nobeep = tempBox->origMode = 0;
 	tempBox->pollTime = tempBox->headerTime = -1;
 	tempBox->BoxNameType = UNDEF;
+   
+
+/* These only apply to cclient stuff, but don't hurt to clear.... */
+   
+        tempBox->keepopen = 0;
+        tempBox->countperiod = 0;
+        tempBox->cycle = 0;
+        tempBox->num_seen_estimate = 0;
+     
+
 }
 
 
@@ -162,6 +146,7 @@ char *parseEnv(str)
    char envStr[MAX_STRING];
    char *ptr,*res;
    char *envValue,*envPtr;
+   char *lbrace;
    
    ptr = str;
    envPtr = envStr;
@@ -171,6 +156,7 @@ char *parseEnv(str)
    {
       if (*ptr == '{')
       {
+	 lbrace = ptr;
 	 ptr++;
 	 while ( (*ptr != '\0') && (*ptr != '}') )
 	 {
@@ -183,8 +169,18 @@ char *parseEnv(str)
 	 
 	 *envPtr = '\0';
 	 envValue = (char *)getenv(envStr);
-	 strcpy(res,envValue);
-	 res+=NEWstrlen(envValue);
+	 
+ 	 /* modified by culver: if getenv() returns 0, then just
+	    copy over the stuff in {...} */
+	 
+	 if (envValue) {
+	   strcpy(res,envValue);
+	   res+=NEWstrlen(envValue);
+	 } else {
+	   *(res++) = *lbrace;
+	   ptr = lbrace+1;
+	 }
+
 	 
       }
       else
@@ -237,7 +233,9 @@ void readBoxfile(boxFile)
 	int inBox;
 	char next[MAX_STRING];
 
-	tempBox.bgName = tempBox.fgName = tempBox.box = tempBox.command = tempBox.audioCmd = tempBox.boxTitle = NULL;
+	tempBox.bgName = tempBox.fgName = tempBox.box = tempBox.command = tempBox.audioCmd = tempBox.boxTitle =tempBox.uname = tempBox.passwd = NULL;
+        tempBox.stream = NULL;
+   
 	clearBox(&tempBox);
 
 
@@ -269,7 +267,9 @@ void readBoxfile(boxFile)
 						tempBox.command,
 						tempBox.audioCmd,
 						tempBox.boxTitle,
-						tempBox.origMode, tempBox.nobeep,tempBox.bgName, tempBox.fgName);
+						tempBox.origMode, tempBox.nobeep,tempBox.bgName, tempBox.fgName,
+					        tempBox.countperiod,
+					        tempBox.keepopen);
 
 #else
 				dumpBox(tempBox);
@@ -303,6 +303,23 @@ void readBoxfile(boxFile)
 		case NEWSBOX_T:
 			tempBox.type = NNTPBOX;
 			break;
+   
+	       case CCLIENT_T:
+#ifdef HAVE_CCLIENT		   
+		    tempBox.type = CCLIENTBOX;
+#else
+		   fprintf(stderr, "program not compiled with -DHAVE_CCLIENT, ignoring box\n");
+		   tempBox.type = NOBOX;
+#endif		   
+  		    break;
+		   
+		   
+	       case COUNTPERIOD_T:
+		     tempBox.countperiod = atoi(next);
+		     break;
+		   
+ 	       case KEEPOPEN_T:
+		      tempBox.keepopen = TRUE;
 
 		case ORIGMODE_T:
 			tempBox.origMode = TRUE;
@@ -362,7 +379,9 @@ void readBoxfile(boxFile)
 				tempBox.command,
 				tempBox.audioCmd,
 				tempBox.boxTitle,
-				tempBox.origMode, tempBox.nobeep,tempBox.bgName,tempBox.fgName);
+				tempBox.origMode, tempBox.nobeep,tempBox.bgName,tempBox.fgName,
+			        tempBox.countperiod,
+                                tempBox.keepopen);
 
 #else
 		dumpBox(tempBox);
@@ -379,7 +398,6 @@ void readBoxfile(boxFile)
 main()
 {
 
-
 	readBoxfile("boxfile.sample");
 }
 
