@@ -17,12 +17,6 @@
  */
 
 
-#ifdef NO_BCOPY
-#define bcopy(x,y,z) memcpy(y,x,z)
-#define bzero(S,len) memset(S,0,len)
-#endif
-
-
 #ifndef _Dyn_h
 #define _Dyn_h
 
