@@ -15,6 +15,9 @@
 
 *******************************************************************************/
 
+#include "xbuffy.h"
+
+#ifdef USE_NNTP
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,7 +30,7 @@
 #include <errno.h>
 #include <sys/stat.h>
 
-#include "xbuffy.h"
+
 
 #define CR		(13)
 #define LF		LINEFEED
@@ -92,16 +95,18 @@ void initNNTP()
 	serverName = getenv("NNTPSERVER");
 
 	if (serverName == NULL)
-		serverName = NEWSSERVER;
+/*		serverName = NEWSSERVER; */
+                serverName = USE_NNTP;
 
 	hp = gethostbyname(serverName);
 
 	if (hp == 0)
 		Fatal("gethostbyname failed");
 
-	bcopy(*hp->h_addr_list, (char *) &server.sin_addr, hp->h_length);
+/*	bcopy(*hp->h_addr_list, (char *) &server.sin_addr, hp->h_length); */
+        memcpy((char *) &server.sin_addr, *hp->h_addr_list, hp->h_length);
 
-	err = connect(sock, (struct aockaddr*)&server, sizeof(server));
+	err = connect(sock, (struct sockaddr*)&server, sizeof(server));
 
 	if (err)
 		Fatal("connect failed");
@@ -162,10 +167,10 @@ int getHeaders(long Article, char **from, char **subject)
 
 		q[-1] = NEWLINE;
 
-		if (strincmp(line, "From:", 5) == 0)
+		if (strncasecmp(line, "From:", 5) == 0)
 			*from = strdup(line);
 
-		if (strincmp(line, "Subject:", 8) == 0)
+		if (strncasecmp(line, "Subject:", 8) == 0)
 			*subject = strdup(line);
 
 
@@ -207,7 +212,7 @@ void readNewsrcEntry(newsBox, firstArt, lastArt)
 		char *tmp;
 		char *backp;
 
-		if (strincmp(line, newsBox->box, NEWstrlen(newsBox->box)) != 0)
+		if (strncasecmp(line, newsBox->box, NEWstrlen(newsBox->box)) != 0)
 			continue;
 
 		tmp = line + NEWstrlen(newsBox->box);
@@ -436,3 +441,5 @@ void main()
 }
 
 #endif
+
+#endif /* HAS_NNTP */
