@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-     Copyright (c) 1994,1995    William Pemberton (wfp5p@virginia.edu)
+     Copyright (c) 1994,1995,1999    William Pemberton (wfp5p@virginia.edu)
 
      The X Consortium, and any party obtaining a copy of these files from
      the X Consortium, directly or indirectly, is granted, free of charge, a
@@ -36,10 +36,7 @@
 #include <ctype.h>
 #include <sys/time.h>
 #include <sys/resource.h>
-#ifndef SUN
-#include <sys/types.h>
 #include <sys/stat.h>
-#endif                          /* SYSV */
 #include "xbuffy.h"
 #ifndef MOTIF
 #include <X11/Intrinsic.h>
@@ -63,6 +60,7 @@
 */
 #endif
 
+
 #include "xbuffy.xbm"
 
 #ifdef WFP_DEBUG
@@ -71,6 +69,7 @@
 
 
 
+
 void CheckBox();
 void TimerBreakPopup();
 int CountUnixMail();
@@ -79,6 +78,11 @@ int makeBoxTitle();
 void initBox();
 Pixel convertColor();
 
+#ifdef HAVE_CCLIENT
+int CountIMAP();
+extern BoxInfo_t *CurrentBox;
+#endif
+
 void ButtonDownHandler();
 void ButtonUpHandler();
 void BreakPopup();
@@ -114,6 +118,8 @@ XtResource resources[] = {
     XtOffset(ApplicationData_t *, mailBoxes), XtRString, 0},
     {"nobeep", "Nobeep", XtRBoolean, sizeof(int),
     XtOffset(ApplicationData_t *, nobeep), XtRString, "FALSE"},
+    {"nofork", "Nofork", XtRBoolean, sizeof(int),
+    XtOffset(ApplicationData_t *, nofork), XtRString, "FALSE"},
     {"horiz", "Horiz", XtRBoolean, sizeof(int),
     XtOffset(ApplicationData_t *, horiz), XtRString, "FALSE"},
     {"command", "Command", XtRString, sizeof(String),
@@ -142,7 +148,7 @@ XtResource resources[] = {
     XtOffset(ApplicationData_t *, fg), XtRString, XtDefaultForeground},
     {XtNbackground, XtCBackground, XtRPixel, sizeof(Pixel),
     XtOffset(ApplicationData_t *, bg), XtRString, XtDefaultBackground},
-#ifdef NNTP
+#ifdef USE_NNTP
     {"newsboxes", "Newsboxes", XtRString, sizeof(String),
     XtOffset(ApplicationData_t *, newsBoxes), XtRString, 0},
 #endif                          /* NNTP */
@@ -151,6 +157,7 @@ XtResource resources[] = {
 
 XrmOptionDescRec options[] = {
     {"-nobeep", "*nobeep", XrmoptionNoArg, "TRUE"},
+    {"-nofork", "*nofork", XrmoptionNoArg, "TRUE"},
     {"-center", "*center", XrmoptionNoArg, "TRUE"},
     {"-fill", "*fill", XrmoptionNoArg, "TRUE"},
     {"-horiz", "*horiz", XrmoptionNoArg, "TRUE"},
@@ -181,16 +188,28 @@ void CheckBox(i)
 
     currentBox = &boxInfo[i];
 
-#ifdef NNTP
-    if (boxInfo[i].type == NNTPBOX)
-    {
+   switch (boxInfo[i].type)
+   {
+      
+     
+#ifdef USE_NNTP
+    case NNTPBOX:
         num = CountNNTP(currentBox, NULL, &beenTouched);
-    }
+        break;
 #endif                          /* NNTP */
-    if (boxInfo[i].type == MAILBOX)
-    {
+      
+#ifdef HAVE_CCLIENT
+    case CCLIENTBOX:
+        num = CountIMAP(currentBox, NULL, &beenTouched);
+        break;
+#endif      
+    case MAILBOX:
         num = CountUnixMail(currentBox, NULL, &beenTouched);
-    }
+        break;
+
+    case NOBOX:break;
+	
+   }
 
     nargs = 0;
     XtSetArg(args[nargs], XtNiconic, &isIcon);
@@ -224,8 +243,10 @@ void CheckBox(i)
         UpdateBoxNumber(currentBox);
     }
 
-
-    XtAppAddTimeOut(app, (currentBox->pollTime * 1000), CheckBox, (XtPointer) i);
+    if (currentBox->pollTime != 0)
+       XtAppAddTimeOut(app,(unsigned long)(currentBox->pollTime * 1000),
+		       (XtTimerCallbackProc) CheckBox, 
+		       (XtPointer) i);
 }
 
 
@@ -289,7 +310,13 @@ void UpdateBoxNumber(box)
    {
       
     offset = ((maxBoxSize+4) - NEWstrlen(amt))/2;
-    ptr = fmtString+ offset;
+      if (offset >0)
+         ptr = fmtString+offset;
+      else 
+      {
+	 ptr = fmtString;
+	 offset = 0;
+      }
     strcpy(ptr,amt);
     ptr = fmtString+NEWstrlen(fmtString);
     *ptr = ' ';
@@ -461,25 +488,26 @@ void PopupHeader(w, i, event, cont)
     DynParanoid(mailHeaders, 1);
 #endif
 
-#ifdef NNTP
-    /* check to see if there is any news before proceeding */
-    if (boxInfo[i].type == NNTPBOX)
-    {
-        number = CountNNTP(&boxInfo[i], mailHeaders, &beenTouched);
-        DynAdd(mailHeaders, "\0");
-        hdrPtr = (char *) DynGet(mailHeaders, 0);
-    }
+     switch (boxInfo[i].type)
+     {
+      
+#ifdef USE_NNTP
+      case NNTPBOX:
+	 number = CountNNTP(&boxInfo[i], mailHeaders, &beenTouched);break;
+#endif 
+      
+#ifdef HAVE_CCLIENT
+    case CCLIENTBOX:
+      number = CountIMAP(&boxInfo[i], mailHeaders, &beenTouched);break;
 #endif
+	
 
-    /* update the number on the box (in case there are new articles) */
-    if (currentBox->type == MAILBOX)
-    {
-        number = CountUnixMail(currentBox, mailHeaders, &beenTouched);
+    case MAILBOX:
+        number = CountUnixMail(currentBox, mailHeaders, &beenTouched);break;
+     }
 
-
-        DynAdd(mailHeaders, "\0");
-        hdrPtr = (char *) DynGet(mailHeaders, 0);
-    }
+    DynAdd(mailHeaders, "\0");
+    hdrPtr = (char *) DynGet(mailHeaders, 0);
 
     /* if the number is different, update it */
     currentBox->n = number;
@@ -672,7 +700,10 @@ int CountUnixMail(mailBox, headerString, beenTouched)
 
     fp = fopen(mailBox->box, "r");
     if (fp == NULL)
+    {
+        fprintf(stderr, "Error: Could not open mailbox %s\n",mailBox->box);
         return 0;
+    }
 
     From[0] = Subject[0] = '\0';
 
@@ -690,7 +721,7 @@ int CountUnixMail(mailBox, headerString, beenTouched)
             while ((c = getc(fp)) != EOF && c != '\n'); /* keep reading */
         }
 
-        if ((!in_header) && (real_from(buffer)))
+        if ((!in_header) && (real_from(buffer,mailBox->type)))
         {
 	    has_CL = FALSE;
             in_header = TRUE;
@@ -726,7 +757,7 @@ int CountUnixMail(mailBox, headerString, beenTouched)
             }
             else if (buffer[0] == LINEFEED)
             {
-#ifdef USE_CONTENT_LENGTH	       
+#ifdef USE_CONTENT_LENGTH
 	       if (has_CL)
 	        fseek(fp,CL,SEEK_CUR);
 #endif	       
@@ -774,7 +805,7 @@ Pixel convertColor(colorname, defValue)
 }
    
 void initBox(box, BoxType, pollTime, headerTime, BoxNameType, command, audioCmd,
-                 title, origMode, nobeep, bgName, fgName)
+                 title, origMode, nobeep, bgName, fgName, countperiod, keepopen)
     char *box;
     BoxType_t BoxType;
     int pollTime;
@@ -787,6 +818,8 @@ void initBox(box, BoxType, pollTime, headerTime, BoxNameType, command, audioCmd,
     Boolean nobeep;
     char *bgName;
     char *fgName;
+    int countperiod;
+    Boolean keepopen;
 
 {
 
@@ -811,7 +844,9 @@ void initBox(box, BoxType, pollTime, headerTime, BoxNameType, command, audioCmd,
     fprintf(stderr, "boxTitle = *%s*\n", title);
     fprintf(stderr, "pollTime = %i  headerTime = %i\n", pollTime, headerTime);
     fprintf(stderr, "nobeep = %i  origMode = %i \n", nobeep, origMode);
-    fprintf(stderr, "nametype = %i\n\n", BoxNameType);
+    fprintf(stderr, "nametype = %i\n", BoxNameType);
+    fprintf(stderr, "countperiod = %i\n", countperiod);
+    fprintf(stderr, "keepopen = %i\n\n",keepopen);
 #endif
 
     tempBox.box = NEWstrdup(box);
@@ -827,7 +862,7 @@ void initBox(box, BoxType, pollTime, headerTime, BoxNameType, command, audioCmd,
 #endif
 
     }
-    if ((pollTime <= 0) || (pollTime >= 3600))
+    if ((pollTime < 0) || (pollTime >= 3600))
         tempBox.pollTime = envPolltime;
     else
         tempBox.pollTime = pollTime;
@@ -873,6 +908,39 @@ void initBox(box, BoxType, pollTime, headerTime, BoxNameType, command, audioCmd,
        
 
     tempBox.box_mtime = tempBox.st_size = 0;
+   
+#ifdef HAVE_CCLIENT
+    if (BoxType == CCLIENTBOX)
+      {
+	tempBox.stream = NULL;
+        tempBox.uname = tempBox.passwd = NULL;
+	tempBox.keepopen = keepopen;
+	
+	 if (tempBox.keepopen) 
+	 {
+	    CurrentBox = &tempBox;
+	  
+	    while (!tempBox.stream)
+	       tempBox.stream = mail_open(NIL, tempBox.box, OP_READONLY);
+	  
+            CurrentBox = NULL;
+	  
+            if (!tempBox.stream) 
+            {
+	      fprintf(stderr,"Can't open IMAP mailbox %s\n",tempBox.box);
+	    }
+	 }
+	 else 
+	 {
+	   tempBox.stream = NULL;
+	 }
+ 	
+	tempBox.num_seen_estimate = 0;
+	tempBox.countperiod = countperiod;
+	tempBox.cycle = countperiod-1;
+      }
+#endif
+
 
     DynAdd(DynBoxObj, &tempBox);
     boxInfo = (BoxInfo_t *) DynGet(DynBoxObj, 0);
@@ -904,7 +972,7 @@ void ParseMailPath()
     while (str != NULL)
     {
         initBox(NEWstrdup(str), MAILBOX, envPolltime, envHeadertime, UNDEF, data.command,
-                data.audioCmd, NULL, data.origMode, data.nobeep,NULL,NULL);
+                data.audioCmd, NULL, data.origMode, data.nobeep,NULL,NULL,0,0);
 
         str = (char *) strtok(NULL, ":, ");
     }
@@ -922,7 +990,7 @@ char *EliminatePath(path)
     return (file);
 }
 
-#ifdef NNTP
+#ifdef USE_NNTP
 void ParseNewsPath()
 {
     char *newsPath = 0;
@@ -1049,6 +1117,7 @@ void Usage()
     printf("  -boxfile <file> filename containing names of mailboxes\n");
     printf("  -horiz          place the boxes horizontally; default: vertical\n");
     printf("  -nobeep         don't ring bell when mail is received\n");
+    printf("  -nofork         don't run in background\n");
     printf("  -names          display full path of mail files in the boxes\n");
     printf("  -shortnames     display names of mail files in the boxes\n");
     printf("  -center         center the names of the boxes\n");
@@ -1056,9 +1125,12 @@ void Usage()
     printf("  -orig           original mode - display all messages in the boxes\n");
     printf("  -command <cmd>  system command to execute when middle button is pushed\n");
     printf("  -mail <files>   specify a mailbox(s) to watch\n");
-#ifdef NNTP
+#ifdef USE_NNTP
     printf("  -news <groups> specify a newsgroup(s) to watch\n");
 #endif                          /* NNTP */
+#ifdef HAVE_CCLIENT
+    printf("C-Client is enabled.\n");
+#endif   
     printf("\n");
     printf("If there are any files specified on the command line, it will\n");
     printf("monitor those mail files, otherwise it will use your MAILPATH\n");
@@ -1067,7 +1139,7 @@ void Usage()
 }
 
 
-void main(argc, argv)
+int main(argc, argv)
     int argc;
     char *argv[];
 
@@ -1094,6 +1166,10 @@ void main(argc, argv)
 #ifdef DEBUG
    char pause_string[10];
 /*  gets(pause_string);*/
+#endif
+   
+#ifdef HAVE_CCLIENT
+#include <c-client/linkage.c>
 #endif
 
     /* initialize program name and version string */
@@ -1173,8 +1249,8 @@ void main(argc, argv)
         }
         else if (strcmp("-news", *argv) == 0)
         {
-#ifndef NNTP
-            fprintf(stderr, "program not compiled with -DNNTP ignoring %s\n", *argv);
+#ifndef USE_NNTP
+            fprintf(stderr, "program not compiled with -DUSE_NNTP ignoring %s\n", *argv);
 #else
             mailArgs = FALSE;
 #endif                          /* !NNTP */
@@ -1199,7 +1275,7 @@ void main(argc, argv)
 
             }
 
-#ifdef NNTP
+#ifdef USE_NNTP
             else
             {
 
@@ -1224,7 +1300,7 @@ void main(argc, argv)
     if (nBoxes == 0)
     {
         ParseMailPath();
-#ifdef NNTP
+#ifdef USE_NNTP
         ParseNewsPath();
 #endif                          /* NNTP */
     }
@@ -1234,7 +1310,7 @@ void main(argc, argv)
     {
         fprintf(stderr, "nothing to watch is specified\n");
         fprintf(stderr, "check $MAILPATH / XBuffy.mailboxes\n");
-#ifdef NNTP
+#ifdef USE_NNTP
         fprintf(stderr, "check $NEWSPATH / XBuffy.newsboxes\n");
 #endif                          /* NNTP */
         Usage();
@@ -1299,10 +1375,15 @@ void main(argc, argv)
         if (boxInfo[i].type == MAILBOX)
             boxInfo[i].n = CountUnixMail(&boxInfo[i], NULL, &dummy);
 
-#ifdef NNTP
+#ifdef USE_NNTP
         if (boxInfo[i].type == NNTPBOX)
             boxInfo[i].n = CountNNTP(&boxInfo[i], NULL, &dummy);
 #endif
+       
+#ifdef HAVE_CCLIENT
+        if (boxInfo[i].type == CCLIENTBOX)
+	    boxInfo[i].n = CountIMAP(&boxInfo[i], NULL, &dummy);
+#endif       
 
         sprintf(name, "box%d", i);
 
@@ -1325,9 +1406,13 @@ void main(argc, argv)
 
 
         XtAddEventHandler(boxInfo[i].w, ButtonPressMask, True,
-                          ButtonDownHandler, &boxInfo[i].boxNum);
-        XtAddEventHandler(boxInfo[i].w, ButtonReleaseMask, True,
-                          ButtonUpHandler, &boxInfo[i].boxNum);
+                          (XtEventHandler) ButtonDownHandler, 
+			  &boxInfo[i].boxNum);
+        XtAddEventHandler(boxInfo[i].w, 
+			  ButtonReleaseMask, 
+			  True,
+                          (XtEventHandler) ButtonUpHandler, 
+			  &boxInfo[i].boxNum);
 
 #else
         nargs = 0;
@@ -1363,11 +1448,13 @@ void main(argc, argv)
 #endif
     XtRealizeWidget(toplevel);
 
-#ifndef DEBUG
-#ifndef NO_SETPRIORITY
+#ifdef HAS_SETPRIORITY
     if (setpriority(PRIO_PROCESS, 0, envPriority) == -1)
         perror("Proirity change Failed");
 #endif
+   
+if (!data.nofork)
+{     
     /* put ourself in the background */
     switch (pid = fork())
     {
@@ -1382,8 +1469,8 @@ void main(argc, argv)
         exit(0);                /* ok its going we can stop now */
         break;
     }
-#else
+}
+else   
     XtAppMainLoop(app);
-#endif
 
 }
